package sun.java2d.loops;

import java.awt.AlphaComposite;
import sun.java2d.SunGraphics2D;
import java.awt.Color;
import sun.awt.image.BufImgSurfaceData;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.Font;
import sun.java2d.pipe.Region;
import java.awt.Composite;
import sun.java2d.SurfaceData;

public class BlitBg extends GraphicsPrimitive
{
    public static final String methodSignature;
    public static final int primTypeID;
    private static RenderCache blitcache;
    
    public static BlitBg locate(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return (BlitBg)GraphicsPrimitiveMgr.locate(BlitBg.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public static BlitBg getFromCache(SurfaceType obj, CompositeType obj2, SurfaceType obj3) {
        Object value = BlitBg.blitcache.get(obj, obj2, obj3);
        if (value != null) {
            return (BlitBg)value;
        }
        BlitBg locate = locate(obj, obj2, obj3);
        if (locate == null) {
            System.out.println("blitbg loop not found for:");
            System.out.println("src:  " + obj);
            System.out.println("comp: " + obj2);
            System.out.println("dst:  " + obj3);
            return locate;
        }
        BlitBg.blitcache.put(obj, obj2, obj3, locate);
        return locate;
    }
    
    protected BlitBg(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(BlitBg.methodSignature, BlitBg.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public BlitBg(long n, SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        super(n, BlitBg.methodSignature, BlitBg.primTypeID, surfaceType, compositeType, surfaceType2);
    }
    
    public native void BlitBg(SurfaceData p0, SurfaceData p1, Composite p2, Region p3, int p4, int p5, int p6, int p7, int p8, int p9, int p10);
    
    @Override
    public GraphicsPrimitive makePrimitive(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
        return new General(surfaceType, compositeType, surfaceType2);
    }
    
    @Override
    public GraphicsPrimitive traceWrap() {
        return new BlitBg.TraceBlitBg(this);
    }
    
    static {
        methodSignature = "BlitBg(...)".toString();
        primTypeID = GraphicsPrimitive.makePrimTypeID();
        BlitBg.blitcache = new RenderCache(20);
        GraphicsPrimitiveMgr.registerGeneral(new BlitBg(null, null, null));
    }
    
    private static class General extends BlitBg
    {
        CompositeType compositeType;
        private static Font defaultFont;
        
        public General(SurfaceType surfaceType, CompositeType compositeType, SurfaceType surfaceType2) {
            super(surfaceType, compositeType, surfaceType2);
            this.compositeType = compositeType;
        }
        
        @Override
        public void BlitBg(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region region, int rgba, int n, int n2, int n3, int n4, int w, int h) {
            ColorModel cm = surfaceData2.getColorModel();
            int hasalpha = rgba >>> 24 != 255;
            if (!cm.hasAlpha() && hasalpha) {
                cm = ColorModel.getRGBdefault();
            }
            SurfaceData data = BufImgSurfaceData.createData(new BufferedImage(cm, cm.createCompatibleWritableRaster(w, h), cm.isAlphaPremultiplied(), null));
            Color o = new Color(rgba, (boolean)hasalpha);
            SunGraphics2D o2 = new SunGraphics2D(data, o, o, BlitBg.General.defaultFont);
            FillRect locate = FillRect.locate(SurfaceType.AnyColor, CompositeType.SrcNoEa, data.getSurfaceType());
            Blit fromCache = Blit.getFromCache(surfaceData.getSurfaceType(), CompositeType.SrcOverNoEa, data.getSurfaceType());
            Blit fromCache2 = Blit.getFromCache(data.getSurfaceType(), this.compositeType, surfaceData2.getSurfaceType());
            locate.FillRect(o2, data, 0, 0, w, h);
            fromCache.Blit(surfaceData, data, AlphaComposite.SrcOver, null, n, n2, 0, 0, w, h);
            fromCache2.Blit(data, surfaceData2, composite, region, 0, 0, n3, n4, w, h);
        }
        
        static {
            BlitBg.General.defaultFont = new Font("Dialog", 0, 12);
        }
    }
    
    private static class TraceBlitBg extends BlitBg
    {
        BlitBg target;
        
        public TraceBlitBg(BlitBg target) {
            super(target.getSourceType(), target.getCompositeType(), target.getDestType());
            this.target = target;
        }
        
        @Override
        public GraphicsPrimitive traceWrap() {
            return this;
        }
        
        @Override
        public void BlitBg(SurfaceData surfaceData, SurfaceData surfaceData2, Composite composite, Region region, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
            GraphicsPrimitive.tracePrimitive(this.target);
            this.target.BlitBg(surfaceData, surfaceData2, composite, region, n, n2, n3, n4, n5, n6, n7);
        }
    }
}
